// PPMsaver
// A class for writing an Image to a stream in PPM format
import java.awt.image.*;
import java.io.IOException;
import LineOutputStream;
import java.util.Hashtable;

public class PPMsaver implements ImageConsumer
{
	int width, height;			// size of image
	byte data[];				// actual data, in PPM style
	int status = RUNNING;			// how are we doing?
	static final int RUNNING = 0;
	static final int OK = 1;
	static final int FAILED = 2;

	// status
	// Returns the status of this image conversion
	int status()
	{
	return status;
	}

	// output
	// Write the data to a ppm format image (raw or ascii)
	void output(LineOutputStream os, boolean raw)
	{
	if (raw) os.puts("P6");
	else os.puts("P3");
	os.puts(width+" "+height);
	os.puts("255");
	if (raw) {
		try { os.write(data); }
		catch(IOException e) { }
		}
	else {
		for(int i=0; i<data.length; i++) {
			int b = data[i];
			if (b < 0) b+=256;
			os.puts(b+" ");
			}
		}
	}

	// imageComplete
	// Called when we have received all bits of the image
	public void imageComplete(int s)
	{
	// System.err.println("imageComplete("+s+") called");
	if ((s & STATICIMAGEDONE) == 0)
		status = FAILED;
	else
		status = OK;
	}

	// setPixels
	// Called to store a rectangle of pixels, who's values are indexes
	// into a colourmap.
	public void setPixels(int x, int y, int w, int h, ColorModel m,
		       byte pixels[], int off, int scansize)
	{
	// System.err.println("setPixels("+x+","+y+","+w+","+h+") called - byte");
	IndexColorModel cm = (IndexColorModel)m;
	for(int i=0; i<w; i++)
		for(int j=0; j<h; j++) {
			int idx = pixels[j*scansize + i + off];
			int pos = ((y+j)*width + (x+i))*3;
			if (idx < 0) idx += 256;
			data[pos]   = (byte)cm.getRed(idx);
			data[pos+1] = (byte)cm.getGreen(idx);
			data[pos+2] = (byte)cm.getBlue(idx);
			}
	}

	// setPixels
	// Called to store a rectangle of pixels, who's values give the
	// RGB values of the pixels.
	public void setPixels(int x, int y, int w, int h, ColorModel m,
		       int pixels[], int off, int scansize)
	{
	// System.err.println("setPixels("+x+","+y+","+w+","+h+") called - int");
	DirectColorModel cm = (DirectColorModel)m;
	for(int i=0; i<w; i++)
		for(int j=0; j<h; j++) {
			int rgb = pixels[j*scansize + i + off];
			int pos = ((y+j)*width + (x+i))*3;
			data[pos] = (byte)cm.getRed(rgb);
			data[pos+1] = (byte)cm.getGreen(rgb);
			data[pos+2] = (byte)cm.getBlue(rgb);
			}
	}

	// setHints
	// Sets various hints about how the image will be delivered. This
	// can be ignored.
	public void setHints(int flags)
	{
	// System.err.println("setHints("+flags+") called");
	}

	// setColorModel
	// Sets the current model that will probably be used for setting
	// pixels. Can be ignored.
	public void setColorModel(ColorModel model)
	{
	// System.err.println("setColorModel("+model+") called");
	}

	// setProperties
	// I don't know what this is supposed to do, so I ignore it
	public void setProperties(Hashtable props)
	{
	// System.err.println("setProperties() called");
	}

	// setDimensions
	// Sets the size of the image. Called before any pixels are sent
	public void setDimensions(int w, int h)
	{
	// System.err.println("setDimensions("+w+","+h+") called");
	width = w;
	height = h;
	data = new byte[w*h*3];
	}
}

